
# q-expansion of newform 325.2.a.d, downloaded from the LMFDB on 05 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 325
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [2], [0], [4], [2], [1], [-2], [-6], [6], [2], [-10], [2], [-6], [-10], [-4], [-2], [6], [2], [4], [6], [6], [-12], [16], [2], [2], [-18], [-2], [-10], [10], [14], [2], [20], [2], [0], [18], [10], [6], [12], [12], [6], [12], [-22], [0], [2], [6], [-16], [12], [-4], [-4], [-22], [-10], [-6], [10], [24], [-2], [-14], [6], [2], [14], [-6], [-2], [-22], [-8], [-4], [22], [18], [-18], [-26], [22], [-30], [-18], [-10], [14], [-34], [10], [-12], [-10], [-6], [10], [18], [-40], [10], [14], [-10], [0], [-14], [-6], [-38], [10], [24], [10], [30], [40], [-24], [10], [18], [-6], [6], [-6], [-22], [6], [26], [22], [-26], [24], [-2], [44], [-14], [-4], [10], [-34], [6], [42], [-2], [-14], [-46], [16], [-38], [6], [0], [2], [30], [6], [20], [22], [-26], [10], [-32], [18], [-34], [6], [12], [-28], [-2], [18], [26], [2], [-8], [-42], [-2], [-42], [50], [46], [32], [34], [38], [-6], [-18], [40], [-12], [-18], [38], [22], [-58], [34], [-48], [-52], [-38], [30], [18], [-24], [-18], [8], [-28], [-18], [-56], [48], [22]]
